package com.xs.core.config.i18n;


import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.LocaleResolver;
import org.springframework.web.servlet.support.RequestContextUtils;

import java.util.Locale;

/**
 * 自定义的国际化拦截器，用于根据请求头中的lang参数来切换国际化语言
 */
public class CustomLocaleChangeInterceptor implements HandlerInterceptor {
    private static final String LANG_HEADER = "lang";

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String lang = request.getHeader(LANG_HEADER);
        if (lang != null && !lang.isEmpty()) {
            LocaleResolver localeResolver = RequestContextUtils.getLocaleResolver(request);
            if (localeResolver != null) {
                Locale locale = parseLocale(lang);
                localeResolver.setLocale(request, response, locale);
            }
        }
        return true;
    }

    private Locale parseLocale(String lang) {
        String[] parts = lang.split("_");
        if (parts.length == 1) {
            return new Locale(parts[0]);
        } else if (parts.length == 2) {
            return new Locale(parts[0], parts[1]);
        } else if (parts.length == 3) {
            return new Locale(parts[0], parts[1], parts[2]);
        }
        return Locale.getDefault();
    }
}
