package com.xs.core.mq.consumer;

import com.alibaba.fastjson2.JSON;
import com.rabbitmq.client.Channel;
import com.xs.core.config.mq.RabbitMQConfig;
import com.xs.core.model.coin.msg.TeamShareMessage;
import com.xs.core.service.team.TeamShareService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * 团队分享产出消息消费者
 */
@Component
@Slf4j
public class TeamShareMessageConsumer {

    @Autowired
    private TeamShareService teamShareService;

    @RabbitListener(queues = RabbitMQConfig.GOLD_COIN_TEAM_SHARE_QUEUE, concurrency = "5")
    public void handleTeamShare(Message m, Channel channel) throws IOException {
        try {
            String s = new String(m.getBody());
            log.info("Received TeamShare message: {}", s);
            TeamShareMessage teamShareMessage = JSON.parseObject(s, TeamShareMessage.class);
            if (teamShareMessage != null) {
                teamShareService.teamShareGoldCoinSettlement(teamShareMessage);
            }
            // 处理完成逻辑
            channel.basicAck(m.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            // 处理失败逻辑 (可以选择重新入队或丢弃消息)
            log.error("Error handleTeamShare message: {}", e.getMessage());
            channel.basicNack(m.getMessageProperties().getDeliveryTag(), false, true);
        }
    }
}
