package com.xs.core.service.Impl.task;

import cn.hutool.core.util.ObjUtil;
import com.xs.core.common.content.UserContext;
import com.xs.core.common.content.UserContextHolder;
import com.xs.core.common.enums.CoinTransactionCategoryEnum;
import com.xs.core.common.enums.CoinTransactionTypeEnum;
import com.xs.core.common.validation.CheckUtils;
import com.xs.core.model.task.entity.TaskInfo;
import com.xs.core.model.task.entity.UserTaskRecord;
import com.xs.core.model.task.req.TaskInfoReq;
import com.xs.core.model.task.resp.TaskInfoResp;
import com.xs.core.service.task.ITaskInfoService;
import com.xs.core.service.task.IUserTaskRecordService;
import com.xs.core.service.task.TaskService;
import com.xs.core.service.wallet.IUserWalletService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

@Service
public class TaskServiceImpl implements TaskService {
    @Autowired
    private ITaskInfoService taskInfoService;

    @Autowired
    private IUserTaskRecordService taskRecordService;

    @Autowired
    private IUserWalletService walletService;


    @Override
    public List<TaskInfoResp> getTaskInfoList(TaskInfoReq req) {
        UserContext context = UserContextHolder.getContext();
        // 获取任务列表
        return taskInfoService.getTaskInfoList(req, context.getId());
    }

    @Override
    @Transactional
    public void handleTask(TaskInfoReq req, Long userId) {
        CheckUtils.throwIfNull(req, "error.task.handle.id.is.null");
        TaskInfo taskInfo = taskInfoService.getById(req.getTaskId());
        CheckUtils.throwIfNull(taskInfo, "error.task.handle.not.exist");
        //查询处理记录
        UserTaskRecord record = taskRecordService.getByTaskId(req.getTaskId(), userId);
        if (ObjUtil.isNull(record)) {
            //说明任务还没开始 添加任务记录
            UserTaskRecord newRecord = new UserTaskRecord();
            newRecord.setTaskId(req.getTaskId());
            newRecord.setUserId(userId);
            newRecord.setStatus(1);
            newRecord.setCreateTime(LocalDateTime.now());
            newRecord.setStartTime(LocalDateTime.now());
            newRecord.setRewardAmount(taskInfo.getGoldCoin());
            taskRecordService.save(newRecord);
        } else {
            //说明任务已经开始 根据任务状态进行处理
            Integer status = record.getStatus();
            if (status == 1) {
                //说明任务已经已经开始 进入下一步
                record.setStatus(2);
                record.setUpdateTime(LocalDateTime.now());
                taskRecordService.updateById(record);
                //领取奖励
                walletService.coinTransaction(CoinTransactionTypeEnum.ADD, CoinTransactionCategoryEnum.TASK_REWARDS, "任务奖励", "任务奖励", new BigDecimal(record.getRewardAmount()), userId);
            }
        }
    }
}
