package com.xs.core.service.Impl.user;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xs.core.common.content.UserContextHolder;
import com.xs.core.common.validation.CheckUtils;
import com.xs.core.model.team.resp.TeamUserResp;
import com.xs.core.model.user.entity.User;
import com.xs.core.mapper.user.UserMapper;
import com.xs.core.service.user.IUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xs.core.utils.InviteCodeGenerator;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.List;

/**
 * <p>
 * tg用户 服务实现类
 * </p>
 *
 * @author xudm
 * @since 2024-12-22
 */
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Override
    public User getInviterByUserId(Long userId) {
        return getBaseMapper().getInviterByUserId(userId);
    }

    @Override
    public List<TeamUserResp> getMyTeamUserList(Long userId, String teamType) {
        return getBaseMapper().getMyTeamUserList(userId, teamType);
    }

    @Override
    public String getInviteCode() {
        Long userId = UserContextHolder.getContext().getId();
        User user = getById(userId);
        CheckUtils.throwIfNull(user, "response.unauthorized");
        String inviteCode = user.getInviteCode();
        if (StrUtil.isBlank(inviteCode)) {
            inviteCode = InviteCodeGenerator.generateInviteCode();
            user.setInviteCode(inviteCode);
            updateById(user);
            return inviteCode;
        }
        return inviteCode;
    }

    @Override
    public Long queryInviterCount(Long id) {
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(User::getReferrerId, id);
        return count(queryWrapper);
    }
}
