package com.xs.core.service.Impl.wallet;

import cn.hutool.core.util.ObjUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xs.core.common.enums.CoinTransactionCategoryEnum;
import com.xs.core.common.enums.CoinTransactionTypeEnum;
import com.xs.core.model.coin.entity.UserCoinTransaction;
import com.xs.core.model.user.entity.User;
import com.xs.core.model.wallet.entity.UserWallet;
import com.xs.core.mapper.wallet.UserWalletMapper;
import com.xs.core.service.coin.IUserCoinTransactionService;
import com.xs.core.service.wallet.IUserWalletService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 用户钱包 服务实现类
 * </p>
 *
 * @author xudm
 * @since 2024-12-22
 */
@Service
public class UserWalletServiceImpl extends ServiceImpl<UserWalletMapper, UserWallet> implements IUserWalletService {
    @Autowired
    private IUserCoinTransactionService userCoinTransactionService;

    @Override
    public UserWallet getWalletByUserId(Long userId) {
        LambdaQueryWrapper<UserWallet> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserWallet::getUserId, userId);
        return getOne(queryWrapper);
    }

    @Override
    @Transactional
    public void coinTransaction(CoinTransactionTypeEnum type, CoinTransactionCategoryEnum category, String reason, String detail, BigDecimal amount, Long userId) {
        // 记录交易记录
        UserCoinTransaction record = new UserCoinTransaction();
        UserWallet userWallet = getWalletByUserId(userId);
        if (ObjUtil.isNull(userWallet)) {
            //如果钱包不存在，则创建
            userWallet = new UserWallet();
            userWallet.setUserId(userId);
            userWallet.setAvailableBalance(BigDecimal.ZERO);
            userWallet.setTotalBalance(BigDecimal.ZERO);
            userWallet.setMoneyBalance(BigDecimal.ZERO);
            userWallet.setUpdatedTime(LocalDateTime.now());
        }
        record.setUserId(userId);
        // 更新用户钱包金币数量
        if (type.equals(CoinTransactionTypeEnum.ADD)) {
            // 增加金币
            BigDecimal added = userWallet.getAvailableBalance().add(amount);
            userWallet.setAvailableBalance(added);
            BigDecimal total = userWallet.getTotalBalance().add(amount);
            userWallet.setTotalBalance(total);
        } else if (type.equals(CoinTransactionTypeEnum.SUBTRACT)) {
            // 减少金币
            userWallet.setAvailableBalance(userWallet.getAvailableBalance().subtract(amount));
        }
        record.setAmount(amount);
        record.setBalance(userWallet.getTotalBalance());
        record.setTransactionType(type.getCode());
        record.setTransactionCategory(category.getCode());
        record.setTransactionReason(reason);
        record.setTransactionDetail(detail);
        saveOrUpdate(userWallet);
        userCoinTransactionService.save(record);
    }

    @Override
    public UserWallet getWalletInfoByUserId(Long id) {
        LambdaQueryWrapper<UserWallet> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserWallet::getUserId, id);
        return getOne(queryWrapper);
    }

    @Override
    public List<UserCoinTransaction> getCoinTransactionByType(Long id, CoinTransactionCategoryEnum category) {
        LambdaQueryWrapper<UserCoinTransaction> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserCoinTransaction::getUserId, id);
        queryWrapper.eq(UserCoinTransaction::getTransactionCategory, category.getCode());
        return userCoinTransactionService.list(queryWrapper);
    }
}
