package com.xs.core.service.Impl.wallet;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjUtil;
import com.xs.core.common.content.UserContext;
import com.xs.core.common.content.UserContextHolder;
import com.xs.core.common.enums.CoinTransactionCategoryEnum;
import com.xs.core.common.enums.CoinTransactionTypeEnum;
import com.xs.core.model.coin.entity.UserCoinSpeedUpgradesRecord;
import com.xs.core.model.coin.entity.UserCoinTransaction;
import com.xs.core.model.wallet.entity.UserWallet;
import com.xs.core.model.wallet.resp.WalletInfoResp;
import com.xs.core.service.coin.IUserCoinSpeedUpgradesRecordService;
import com.xs.core.service.wallet.IUserWalletService;
import com.xs.core.service.wallet.WalletService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;

@Service
public class WalletServiceImpl implements WalletService {
    @Autowired
    private IUserWalletService walletService;

    @Autowired
    private IUserCoinSpeedUpgradesRecordService speedUpgradesRecordService;

    @Override
    public WalletInfoResp getWalletInfo() {
        UserContext context = UserContextHolder.getContext();
        //查询用户钱包信息
        CompletableFuture<UserWallet> userWalletFuture = CompletableFuture.supplyAsync(() -> walletService.getWalletByUserId(context.getId()));
        //查询用户交易信息
        CompletableFuture<BigDecimal> totalProfitPerHourFuture = CompletableFuture.supplyAsync(() -> {
            List<UserCoinTransaction> totalProfitPerHour = walletService.getCoinTransactionByType(context.getId(), CoinTransactionCategoryEnum.PRODUCT);
            return assistCalculate(totalProfitPerHour);
        });
        //邀请奖励
        CompletableFuture<BigDecimal> shareRewardsFuture = CompletableFuture.supplyAsync(() -> {
            List<UserCoinTransaction> shareRewards = walletService.getCoinTransactionByType(context.getId(), CoinTransactionCategoryEnum.SHARE_REWARDS);
            return assistCalculate(shareRewards);
        });
        //任务奖励
        CompletableFuture<BigDecimal> taskRewardsFuture = CompletableFuture.supplyAsync(() -> {
            List<UserCoinTransaction> taskRewards = walletService.getCoinTransactionByType(context.getId(), CoinTransactionCategoryEnum.TASK_REWARDS);
            return assistCalculate(taskRewards);
        });
        //团队奖励
        CompletableFuture<BigDecimal> teamSharingFuture = CompletableFuture.supplyAsync(() -> {
            List<UserCoinTransaction> teamSharing = walletService.getCoinTransactionByType(context.getId(), CoinTransactionCategoryEnum.TEAM_SHARING);
            return assistCalculate(teamSharing);
        });
        //空投
        CompletableFuture<BigDecimal> airdropsFuture = CompletableFuture.supplyAsync(() -> {
            List<UserCoinTransaction> teamSharing = walletService.getCoinTransactionByType(context.getId(), CoinTransactionCategoryEnum.RESOURCE_PLACEMENT);
            return assistCalculate(teamSharing);
        });
        //查询用户升级信息
        CompletableFuture<UserCoinSpeedUpgradesRecord> speedUpgradesRecordFuture = CompletableFuture.supplyAsync(() -> speedUpgradesRecordService.getSpeedUpgradesRecordByUserId(context.getId()));
        CompletableFuture.allOf(userWalletFuture, totalProfitPerHourFuture, shareRewardsFuture, taskRewardsFuture, teamSharingFuture, airdropsFuture);

        return new WalletInfoResp(userWalletFuture.join(), totalProfitPerHourFuture.join(), shareRewardsFuture.join(), taskRewardsFuture.join(), teamSharingFuture.join(), airdropsFuture.join(), speedUpgradesRecordFuture.join());
    }

    /**
     * 辅助计算
     *
     * @param transactions 交易记录
     */
    private BigDecimal assistCalculate(List<UserCoinTransaction> transactions) {
        if (CollUtil.isNotEmpty(transactions)) {
            return transactions.stream().filter(trans -> ObjUtil.isNotNull(trans.getAmount()))
                    .map(transaction -> {
                        if (Objects.equals(transaction.getTransactionType(), CoinTransactionTypeEnum.ADD.getCode())) {
                            // 增加
                            return transaction.getAmount();
                        } else if (Objects.equals(transaction.getTransactionType(), CoinTransactionTypeEnum.SUBTRACT.getCode())) {
                            // 消耗（负值）
                            return transaction.getAmount().negate();
                        } else {
                            // 未知类型，当作0处理
                            return BigDecimal.ZERO;
                        }
                    })
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
        }
        return BigDecimal.ZERO;
    }
}
