package com.xs.core.service.wallet;

import com.xs.core.common.enums.CoinTransactionCategoryEnum;
import com.xs.core.common.enums.CoinTransactionTypeEnum;
import com.xs.core.model.coin.entity.UserCoinTransaction;
import com.xs.core.model.wallet.entity.UserWallet;
import com.baomidou.mybatisplus.extension.service.IService;

import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 用户钱包 服务类
 * </p>
 *
 * @author xudm
 * @since 2024-12-22
 */
public interface IUserWalletService extends IService<UserWallet> {
    UserWallet getWalletByUserId(Long userId);

    /**
     * 用户金币交易
     *
     * @param type     交易类型 增加/减少
     * @param category 交易类型
     * @param reason   交易原因
     * @param detail   交易详情
     * @param amount   交易数量
     * @param userId   用户id
     */
    void coinTransaction(CoinTransactionTypeEnum type, CoinTransactionCategoryEnum category, String reason, String detail, BigDecimal amount, Long userId);

    /**
     * 获取钱包信息
     *
     * @param id 用户id
     */
    UserWallet getWalletInfoByUserId(Long id);

    /**
     * 获取用户交易记录
     *
     * @param id       用户id
     * @param category 交易类型
     */
    List<UserCoinTransaction> getCoinTransactionByType(Long id, CoinTransactionCategoryEnum category);
}
